## Random Sampling Function Script File ##

# library(tidyverse)   # functions to work with tibbles
# library(lubridate)   # makes working with dates easier


source(paste0(rstudioapi::getActiveProject(),"/R Functions/function_predictionModels.R"))

geomean = function(x, na.rm=TRUE){
  exp(sum(log(x[x > 0]), na.rm=na.rm) / length(x))
}

AqSampleFishModelPairing <- function(aqData, fishModels, bylocationColumn=NULL, aqYearColumn=Year, aqOverlapYrs=5){

  # FUNCTION VARIABLES
  # aqData           - dataframe of Aq   data containing columns: "locationColumn", Result, Year, Unit
  # fishModels       - dataframe of selected Fish Std Length [Hg] Models containing columns: "locationColumn", Stdz350, Year, Unit
  # bylocationColumn - column of location names. LOcation must be shared between Fish & Aq to be paired
  # byaqYearColumn   - column of years designating the year the aqueous sample was taken
  # aqOverlapYrs     - number years prior Fish sample available to draw aq random sample
  
  # bylocationColumn <- quo(Subarea) - to test function line-wise
  locationColumn <- enquo(bylocationColumn)
  
  # aqYearColumn <- quo(Year) - to test function line-wise
  aqYearColumn <- enquo(aqYearColumn)
           
  
  # START PAIRING
  Locations <- fishModels %>% distinct(!!locationColumn) %>% pull
 
  FishAqResults <- NULL
  for(location in Locations){
    # location <- Locations[1]   # - to test function line-wise
    
    fish_loc <- fishModels %>% 
      filter(!!locationColumn == location)
    
    Years <- fish_loc %>% distinct(Year) %>% pull
    
    tempFishAqResults <- NULL
    for(yr in Years){
      # yr <- Years[1]   # - to test function line-wise
      
      ## Gather Fish Samples ##
      fish_locYr <- fish_loc %>% 
        filter(Year == yr)
    
      

      ## Gather Aq Samples ##
      
      aqYrSamp <- aqData %>% 
        filter(!!locationColumn == location,
               !!aqYearColumn <= yr & !!aqYearColumn > yr-aqOverlapYrs) # Includes year both fish & Aq are sampled plus aqOverlapYrs-1 previous years of Aq data
      
      #data check that Aq data exists for subarea - if not go to next year
      if(is.null(aqYrSamp) | nrow(aqYrSamp) < 1){next}
      
      # Calc Aq Concs
      aqSampleYrs <- paste(aqYrSamp %>% distinct(!!aqYearColumn) %>% pull %>% sort, collapse=", ") # Collapse Aq Sample Years for storage in dataframe
            
      tempAqSamp <- tibble(aq_sampleYears = aqSampleYrs,
                           aq_n = nrow(aqYrSamp),
                           aq_Mean_pool      = mean(aqYrSamp$Result),
                           aq_Mean_byYear    = aqYrSamp %>% group_by(!!aqYearColumn) %>% summarise(Avg= mean(Result)) %>% summarise(Mean=mean(Avg), .groups = 'drop') %>% pull(Mean),
                           aq_Wt.Mean        = aqYrSamp %>% group_by(!!aqYearColumn) %>% summarise(Avg= mean(Result), n=n()) %>% summarise(WtMean=weighted.mean(Avg, n), .groups = 'drop') %>% pull(WtMean),
                           aq_Geomean_pool   = geomean(aqYrSamp$Result),
                           aq_Geomean_byYear = aqYrSamp %>% group_by(!!aqYearColumn) %>% summarise(GeoAvg= geomean(Result)) %>% summarise(GeoMean=geomean(GeoAvg), .groups = 'drop') %>% pull(GeoMean),
                           aq_Median_pool    = median(aqYrSamp$Result),
                           aq_Median_byYear  = aqYrSamp %>% group_by(!!aqYearColumn) %>% summarise(Medi= median(Result)) %>% summarise(Median=median(Medi), .groups = 'drop') %>% pull(Median),
                           aq_Unit = unique(aqData$Unit)) 
      
      # Store Results
      tempFishAqResults <- rbind(tempFishAqResults, cbind(fish_locYr, tempAqSamp))
     
    }
    FishAqResults <- rbind(FishAqResults, tempFishAqResults)
  }
  return(FishAqResults) 
}

